/*
 * Copyright 2012 JBoss Inc
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.jbpm.console.ng.client;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.inject.Inject;

import com.google.gwt.animation.client.Animation;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.MultiWordSuggestOracle;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.SuggestBox.DefaultSuggestionDisplay;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import org.jboss.errai.ioc.client.api.AfterInitialization;
import org.jboss.errai.ioc.client.api.EntryPoint;
import org.jboss.errai.ioc.client.container.IOCBeanDef;
import org.jboss.errai.ioc.client.container.IOCBeanManager;
import org.jbpm.console.ng.ht.client.resources.ShowcaseResources;
import org.uberfire.client.mvp.AbstractWorkbenchPerspectiveActivity;
import org.uberfire.client.mvp.ActivityManager;
import org.uberfire.client.mvp.Command;
import org.uberfire.client.mvp.PlaceManager;
import org.uberfire.client.workbench.widgets.menu.MenuFactory;
import org.uberfire.client.workbench.widgets.menu.MenuItem;
import org.uberfire.client.workbench.widgets.menu.MenuPosition;
import org.uberfire.client.workbench.widgets.menu.Menus;
import org.uberfire.client.workbench.widgets.menu.WorkbenchMenuBarPresenter;
import org.uberfire.security.Identity;
import org.uberfire.security.Role;
import org.uberfire.shared.mvp.PlaceRequest;
import org.uberfire.shared.mvp.impl.DefaultPlaceRequest;

/**
 *
 */
@EntryPoint
public class ShowcaseEntryPoint {

    @Inject
    private PlaceManager placeManager;
    @Inject
    private WorkbenchMenuBarPresenter menubar;
    @Inject
    private ActivityManager activityManager;
    @Inject
    private IOCBeanManager iocManager;
    private String[] menuItems = new String[]{
        "Authoring",
        "Tasks",
        "Process Definitions",
        "Process Instances", //        "Users and Groups",
    //        "Jobs",
    //   "Kie Sessions List",
    };
    @Inject
    public Identity identity;

    @AfterInitialization
    public void startApp() {
        loadStyles();
        setupMenu();
        hideLoadingPopup();
        registerDoAction();
    }

    private void loadStyles() {
        //Ensure CSS has been loaded
        ShowcaseResources.INSTANCE.showcaseCss().ensureInjected();
        //RoundedCornersResource.INSTANCE.roundCornersCss().ensureInjected();
    }

    private void registerDoAction() {
//        KeyPressHandler keyPressHandler = new KeyPressHandler() {
//            public void onKeyPress( KeyPressEvent event ) {
//
//                if ( event.getUnicodeCharCode() == 160 && event.isAltKeyDown() ) {
//                    final DialogBox dialogBox = createDialogBox();
//                    dialogBox.setGlassEnabled( true );
//                    dialogBox.setAnimationEnabled( true );
//                    dialogBox.center();
//                    dialogBox.show();
//                    actionText.setFocus( true );
//                }
//                if ( event.isControlKeyDown() && event.getUnicodeCharCode() == 116 ) {
//                    PlaceRequest placeRequestImpl = new DefaultPlaceRequest( "Quick New Task" );
//                    placeManager.goTo( placeRequestImpl );
//                }
//
//                if ( event.isControlKeyDown() && event.getUnicodeCharCode() == 104 ) {
//                    PlaceRequest placeRequestImpl = new DefaultPlaceRequest( "Home Perspective" );
//                    placeManager.goTo( placeRequestImpl );
//                }
//
//            }
//        };
//
//        RootPanel.get().addDomHandler( keyPressHandler, KeyPressEvent.getType() );
    }

    private void setupMenu() {

        final AbstractWorkbenchPerspectiveActivity defaultPerspective = getDefaultPerspectiveActivity();

        final Menus menus = MenuFactory
                .newTopLevelMenu("Home")
                .respondsWith(new Command() {
            @Override
            public void execute() {
                if (defaultPerspective != null) {
                    placeManager.goTo(new DefaultPlaceRequest(defaultPerspective.getIdentifier()));
                } else {
                    Window.alert("Default perspective not found.");
                }
            }
        })
                .endMenu()
                .newTopLevelMenu("Authoring")
                .withItems(getAuthoringViews())
                .endMenu()
                .newTopLevelMenu("Process Management")
                .withItems(getProcessMGMTViews())
                .endMenu()
                .newTopLevelMenu("Work")
                .withItems(getWorkViews())
                .endMenu()
                .newTopLevelMenu("BAM")
                .withItems(getBAMViews())
                .endMenu()
                .newTopLevelMenu("Logout")
                .respondsWith(new Command() {
            @Override
            public void execute() {
                redirect(GWT.getModuleBaseURL() + "uf_logout");
            }
        })
                .endMenu()
                .newTopLevelMenu(identity.getName())
                .position(MenuPosition.RIGHT)
                .withItems(getRoles())
                .endMenu()
                .build();

        menubar.aggregateWorkbenchMenus(menus);
    }

    private List<? extends MenuItem> getRoles() {
        final List<MenuItem> result = new ArrayList<MenuItem>(identity.getRoles().size());
        for (final Role role : identity.getRoles()) {
            result.add(MenuFactory.newSimpleItem(role.getName()).endMenu().build().getItems().get(0));
        }

        return result;
    }

    private List<? extends MenuItem> getAuthoringViews() {
        final List<MenuItem> result = new ArrayList<MenuItem>(1);

        result.add(MenuFactory.newSimpleItem("Process Authoring").respondsWith(new Command() {
            @Override
            public void execute() {
                placeManager.goTo(new DefaultPlaceRequest("Authoring"));
            }
        }).endMenu().build().getItems().get(0));


        return result;
    }

    private List<? extends MenuItem> getProcessMGMTViews() {
        final List<MenuItem> result = new ArrayList<MenuItem>(2);
        

        result.add(MenuFactory.newSimpleItem("Process Definitions").respondsWith(new Command() {
            @Override
            public void execute() {
                placeManager.goTo(new DefaultPlaceRequest("Process Definitions"));
            }
        }).endMenu().build().getItems().get(0));

        result.add(MenuFactory.newSimpleItem("Process Instances").respondsWith(new Command() {
            @Override
            public void execute() {
                placeManager.goTo(new DefaultPlaceRequest("Process Instances"));
            }
        }).endMenu().build().getItems().get(0));


        return result;
    }
    
    private List<? extends MenuItem> getWorkViews() {
        final List<MenuItem> result = new ArrayList<MenuItem>(2);
        

        result.add(MenuFactory.newSimpleItem("Tasks - Calendar View").respondsWith(new Command() {
            @Override
            public void execute() {
                placeManager.goTo(new DefaultPlaceRequest("Tasks"));
            }
        }).endMenu().build().getItems().get(0));

        result.add(MenuFactory.newSimpleItem("Tasks - Grid View").respondsWith(new Command() {
            @Override
            public void execute() {
                placeManager.goTo(new DefaultPlaceRequest("Grid Tasks List"));
            }
        }).endMenu().build().getItems().get(0));


        return result;
    }

    private List<? extends MenuItem> getBAMViews() {
        final List<MenuItem> result = new ArrayList<MenuItem>(1);
        result.add(MenuFactory.newSimpleItem("Process Dashboard").respondsWith(new Command() {
            @Override
            public void execute() {
                Window.open("http://localhost:8080/dashbuilder/workspace/jbpm-dashboard", "_blank", "");
            }
        }).endMenu().build().getItems().get(0));
        return result;
    }

    private AbstractWorkbenchPerspectiveActivity getDefaultPerspectiveActivity() {
        AbstractWorkbenchPerspectiveActivity defaultPerspective = null;
        final Collection<IOCBeanDef<AbstractWorkbenchPerspectiveActivity>> perspectives = iocManager.lookupBeans(AbstractWorkbenchPerspectiveActivity.class);
        final Iterator<IOCBeanDef<AbstractWorkbenchPerspectiveActivity>> perspectivesIterator = perspectives.iterator();
        outer_loop:
        while (perspectivesIterator.hasNext()) {
            final IOCBeanDef<AbstractWorkbenchPerspectiveActivity> perspective = perspectivesIterator.next();
            final AbstractWorkbenchPerspectiveActivity instance = perspective.getInstance();
            if (instance.isDefault()) {
                defaultPerspective = instance;
                break outer_loop;
            } else {
                iocManager.destroyBean(instance);
            }
        }
        return defaultPerspective;
    }

    private List<AbstractWorkbenchPerspectiveActivity> getPerspectiveActivities() {

        //Get Perspective Providers
        final Set<AbstractWorkbenchPerspectiveActivity> activities = activityManager.getActivities(AbstractWorkbenchPerspectiveActivity.class);

        //Sort Perspective Providers so they're always in the same sequence!
        List<AbstractWorkbenchPerspectiveActivity> sortedActivities = new ArrayList<AbstractWorkbenchPerspectiveActivity>(activities);
        Collections.sort(sortedActivities,
                new Comparator<AbstractWorkbenchPerspectiveActivity>() {
            @Override
            public int compare(AbstractWorkbenchPerspectiveActivity o1,
                    AbstractWorkbenchPerspectiveActivity o2) {
                return o1.getPerspective().getName().compareTo(o2.getPerspective().getName());
            }
        });

        return sortedActivities;
    }
    //Fade out the "Loading application" pop-up

    private void hideLoadingPopup() {
        final Element e = RootPanel.get("loading").getElement();

        new Animation() {
            @Override
            protected void onUpdate(double progress) {
                e.getStyle().setOpacity(1.0 - progress);
            }

            @Override
            protected void onComplete() {
                e.getStyle().setVisibility(Style.Visibility.HIDDEN);
            }
        }.run(500);
    }

    public static native void redirect(String url)/*-{
     $wnd.location = url;
     }-*/;

}
